package cern.jira.emailhandler;

import java.io.IOException;
import java.util.Map;
import com.atlassian.mail.MailUtils;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Part;

import org.apache.log4j.Category;
import org.ofbiz.core.entity.GenericValue;

import com.atlassian.jira.service.util.ServiceUtils;
import com.atlassian.jira.service.util.handler.AbstractMessageHandler;


/**
 * This is the main handler class of the JIRA Advanced Mail Handler. 
 * <p>It permits to create a new issue, or add a comment
 * to an existing issue, from an incoming message. If the recipient or subject
 * contains a project key the message is added as a comment to that
 * issue; in this case, many of the issue options can be specified directly
 * in the email body. If no project key is found, a new issue is created in the
 * specified project.
 *
 * @author <a href="http://draffo.web.cern.ch">Daniele Raffo</a> 
 * on the basis of the CreateOrCommentHandler class, copyright (c) 2002-2006 Atlassian
 * @version 3.0 (19-3-2009)
 */
public class AdvancedCreateOrCommentHandler extends AbstractMessageHandler {
	
    private static final Category log = Category.getInstance(AdvancedCreateOrCommentHandler.class);

    /** Default project where new issues are created. */
    public String defaultProjectKey;
    /** Default type for new issues. */
    public String defaultIssueType;
    /** If set (to anything), quoted text is removed from comments. */
    public String stripquotes;
	/** Regex for JIRA email address. */
    public String jiraEmail;
	/** Regex for alias of JIRA email address. */    
    public String jiraEmailAlias;
    
    private static final String KEY_PROJECT = "project";
    private static final String KEY_ISSUETYPE = "issuetype";
    private static final String KEY_QUOTES = "stripquotes";
    private static final String FALSE = "false";
    private static final String KEY_JIRAEMAIL = "jiraemail";
    private static final String KEY_JIRAALIAS = "jiraalias";

    
    public void init(Map params) {
        log.debug("AdvancedCreateOrCommentHandler.init(params: " + params + ")");

        super.init(params);

        if (params.containsKey(KEY_PROJECT)) {
            defaultProjectKey = (String) params.get(KEY_PROJECT);
        }

        if (params.containsKey(KEY_ISSUETYPE)) {
            defaultIssueType = (String) params.get(KEY_ISSUETYPE);
        }

        if (params.containsKey(KEY_QUOTES)) {
            stripquotes = (String) params.get(KEY_QUOTES);
        }

        if (params.containsKey(KEY_JIRAEMAIL)) {
             jiraEmail = (String) params.get(KEY_JIRAEMAIL);
        }

        if (params.containsKey(KEY_JIRAALIAS)) {
            jiraEmailAlias = (String) params.get(KEY_JIRAALIAS);
        } else {
        	jiraEmailAlias = jiraEmail;
        }

        log.debug("Params: " + defaultProjectKey + " - " + defaultIssueType + " - " + stripquotes + " - " + jiraEmail + " - " + jiraEmailAlias);
    } 

    
	public boolean handleMessage(Message message) throws MessagingException {
        log.debug("AdvancedCreateOrCommentHandler.handleMessage");

        if (!canHandleMessage(message)) {
            return deleteEmail;
        }

        String subject = message.getSubject();
        GenericValue issue = ServiceUtils.findIssueInString(subject);
        
        IssueDescriptor issueDescriptor = MessageParser.parse(message, 
        		new String[] {jiraEmail, jiraEmailAlias});
        
        if (issue == null) {
            // If we cannot find the issue from the subject of the e-mail message
            // try finding the issue using the in-reply-to message id of the e-mail message
            issue = getAssociatedIssue(message);
        }

        // if we have found an associated issue
        if (issue != null) {
            boolean doDelete = false;

            // add the message as a comment to the issue
            if ((stripquotes == null) || FALSE.equalsIgnoreCase(stripquotes)) { // if stripquotes not defined in setup
                FullCommentHandler fc = new FullCommentHandler();
                fc.setErrorHandler(this.getErrorHandler());
                fc.init(params);
                doDelete = fc.handleMessage(message); // get message with quotes
            } else {
                NonQuotedCommentHandler nq = new NonQuotedCommentHandler();
                nq.setErrorHandler(this.getErrorHandler());
                nq.init(params);
                doDelete = nq.handleMessage(message); //get message without quotes
            }
            return doDelete;
        } else { // no issue found, so create new issue in default project
            AdvancedCreateIssueHandler createIssueHandler = new AdvancedCreateIssueHandler();
            createIssueHandler.setIssueDescriptor(issueDescriptor);
            createIssueHandler.setErrorHandler(this.getErrorHandler());
            createIssueHandler.init(params);
            return createIssueHandler.handleMessage(message);
        }
    }



    /**
     * Attaches HTML parts. 
     * Comments never wish to keep HTML parts that are not attachments, as they extract the plain text
     * part and use that as content. This method therefore is hard wired to always return false.
     *
     * @param part  the HTML part being processed
     * @return  always false
     * @throws  MessagingException
     * @throws  IOException
     */
    protected boolean attachHtmlParts(final Part part) throws MessagingException, IOException
    {
        return false;
    }

    /**
     * Attaches plaintext parts. Plain text parts must be kept if they are not empty.
     *
     * @param part  the plain text part being tested
     * @return  true if content is not empty and must be attached
     */
    protected boolean attachPlainTextParts(final Part part) throws MessagingException, IOException
    {
        return !MailUtils.isContentEmpty(part);
    }
    
}
